@@ -41,6 +41,19 @@ sub attack {
 sub difficulty { return 'very easy' }
 
 
+#
+# my @where = $ai->move_armies;
+#
+# See pod in Games::Risk::AI for information on the goal of this method.
+#
+# This implementation will not move any armies at all.
+#
+sub move_armies {
+    my ($self) = @_;
+    return;
+}
+
+
 #
 # my @where = $ai->place_armies($nb, [$continent]);
 #
@@ -121,6 +134,8 @@ C<Games::Risk::AI>):
 
 =item * difficulty()
 
+=item * move_armies()
+
 =item * place_armies()
 
 =back
